# Created:01-14-01
# By Jeff Connelly

# The Unifont project is located at http://czyborra.com/unifont/.

# Convert the slow UNIFONT.HEX file format into an ordered (no char value
# field) and fixed-field format. Speeds up access from O(n) to O(1).
#
# - Two hexadecimal digits are converted to one byte        (halfs space)
# - Half-width characters have their nonexistant half       (speed: O(1))
#   filled with nulls
# - Character U+xxxx is located at xxxx * 64                (speed: O(1))
#   Good for random access.
#
# This greatly improves the access time, while shrinking size a bit or two:
#
#   UNIFONT  HEX     1,354,280  01-14-01  3:11p Unifont.hex
#   UNIFONT  BIN     1,218,112  01-15-01 11:11a Unifont.bin
#                      136,168 decrease in size
# To save space, unifont.hex (the original file from czyborra) was not
# included.

use strict;
use warnings;

our @glyphs;
our $default_glyph;
$default_glyph = ("\x42" x 16) . ("\0" x 16);

# Uncomment if you want to decode .bin back to .hex format
#eval(join "", <DATA>)

open(HEX, "<unifont.hex") || die;
while(<HEX>)
{
    chomp;
    next if $_ eq "";

    my ($code, $glyph) = split /:/;

    $glyph = pack 'H*', $glyph;                 # Encode hex into bytes
    $glyph .= "\0" x (32 - length $glyph);      # Pad half-widths with null
    die if length($glyph) != 32;

    # Array, not hash, because arrays are ordered.
    $glyphs[oct "0x$code"] = $glyph;
}
close(HEX);

open(BIN, ">unifont.bin") || die;
binmode(BIN);
for my $i (0..@glyphs)
{
    my $glyph = $glyphs[$i];
    if (!defined($glyphs[$i])) { $glyph = $default_glyph }
    if (tell(BIN) != $i * 32)
    {
        die "bad pos: ", tell(BIN), " for $i", " len = ",
            length($glyph);
    }
    print BIN $glyph;
}

__DATA__
# Convert UNIFONT.BIN back to UNIFONT.HEX

open(BIN, "<unifont.bin") || die;
binmode(BIN);
open(HEX, ">1unifont.hex") || die;
$/ = \64;
binmode(BIN);
while(<BIN>)
{
    next if $_ eq "\0" x 64;                    # Space filler
    printf HEX "%.4X:", $i;
    my $glyph = unpack("H64", $_);
    if (substr($glyph, 32, 64) eq "0" x 32)     # Half-width character
    {
        substr($glyph, 32, 64) = "";
    }
    print HEX "$glyph\n";
    ++$i;
}
close(HEX);
close(BIN);

